/*--------------------------------------------------------------
    trackpeerd - peer tracking agent
    basic communication module    comm.c
      Written by  H.Goto, Jan 2005
      Modified by H.Goto, Mar 2005
--------------------------------------------------------------*/

/*----------------------------------------------------------------------
  Copyright (C) 2005  Hideaki Goto

  All Rights Reserved.

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files (the
  "Software"), to deal in the Software without restriction, including
  without limitation the rights to use, copy, modify, merge, publish,
  distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to
  the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR
  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name(s) of the copyright
  holder(s) shall not be used in advertising or otherwise to promote
  the sale, use or other dealings in this Software without prior
  written authorization of the copyright holder(s).
----------------------------------------------------------------------*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/socket.h>
#include	<arpa/inet.h>
#include	<sys/time.h>
#include	<sys/types.h>
#include	<netinet/in.h>
#include	<unistd.h>

#include	"comm.h"




/*--------------------------------------------------------------
    Convert Addr:Port or Addr.Port form of string to values
--------------------------------------------------------------*/

int get_addr_port(char *addrstr, in_addr_t *addr, uint16_t *port){
	char	*p;
	/* accept AAA.AAA.AAA.AAA:Port and AAA.AAA.AAA.AAA.Port */
	if ( NULL == (p = strrchr(addrstr,':')) ){
		if ( NULL == (p = strrchr(addrstr,'.')) )  return(-1);
	}
	*p++ = '\0';
	*port = (uint16_t)((unsigned int)atoi(p));
	if ( 0 >= inet_pton(AF_INET,addrstr,addr) )  return(-1);
	return(0);
}




/*--------------------------------------------------------------
    Convert Addr/Netmask form of string to values
--------------------------------------------------------------*/

int get_addr_mask(char *addrstr, in_addr_t *addr, in_addr_t *mask){
	char	*p;
	*mask = htonl(0xffffffff);
	if ( NULL != (p = strrchr(addrstr,'/')) ){
		*p++ = '\0';
		if ( NULL != strrchr(p,'.') ){
			if ( 0 >= inet_pton(AF_INET,p,mask) )  return(-1);
		}
		else{	*mask = htonl((unsigned int)0xffffffff << (32-atoi(p)));
			if ( atoi(p) == 0 )  *mask = 0;
		}
	}
	if ( 0 >= inet_pton(AF_INET,addrstr,addr) )  return(-1);
	return(0);
}




/*--------------------------------------------------------------
    Receive/Send binary data
--------------------------------------------------------------*/

int ReceiveData(int sock, char *buf, int len, time_t timeout){
	int	recvcount = 0;
	int	rv;
	char	c;
	fd_set	sockSet;
	int	maxdesc;
	struct timeval	seltimeout;

	if ( len <= 0 )  return(-1);	/* incorrect parameter */
	for (;;){
		FD_ZERO(&sockSet);
		FD_SET(sock, &sockSet);
		maxdesc = sock;

		seltimeout.tv_sec = timeout;
		seltimeout.tv_usec = 0;
		rv = select(maxdesc +1, &sockSet, NULL, NULL, &seltimeout);
		if ( rv == 0 ){		/* timeout occurred */
			return(-2);
		}

		/* This code is of no use since only one socket is watched. */
		if ( ! FD_ISSET(sock, &sockSet) )  continue;

		if ( 0 > (rv = recv(sock, &c, 1, 0)) ){
			return(-1);	/* read error */
		}
		if ( rv == 0 )  break;
		if ( recvcount < len ){
			buf[recvcount++] = c;
		}
	}
	return(recvcount);
}


int SendData(int sock, char *buf, int len, time_t timeout){
	int	sentcount;
	int	rv;
	fd_set	sockSet;
	int	maxdesc;
	struct timeval	seltimeout;
	for ( sentcount=0 ; sentcount<len ; ){
		FD_ZERO(&sockSet);
		FD_SET(sock, &sockSet);
		maxdesc = sock;

		seltimeout.tv_sec = timeout;
		seltimeout.tv_usec = 0;
		rv = select(maxdesc +1, NULL, &sockSet, NULL, &seltimeout);
		if ( rv == 0 ){		/* timeout occurred */
			return(-2);
		}

		/* This code is of no use since only one socket is watched. */
		if ( ! FD_ISSET(sock, &sockSet) )  continue;

		if ( send(sock, buf, 1, 0) != 1 )  return(-1);
		buf++;  sentcount++;
	}
	return(sentcount);
}




/*--------------------------------------------------------------
    Receive/Send a message line
--------------------------------------------------------------*/

int ReceiveLine(int sock, char *msg, int buflen, time_t timeout){
	int	recvcount = 0;
	int	rv;
	char	c;
	fd_set	sockSet;
	int	maxdesc;
	struct timeval	seltimeout;

	if ( buflen <= 0 )  return(-1);	/* incorrect parameter */
	buflen--;	/* make a room for terminator */
	for (;;){
		FD_ZERO(&sockSet);
		FD_SET(sock, &sockSet);
		maxdesc = sock;

		seltimeout.tv_sec = timeout;
		seltimeout.tv_usec = 0;
		rv = select(maxdesc +1, &sockSet, NULL, NULL, &seltimeout);
		if ( rv == 0 ){		/* timeout occurred */
			return(-2);
		}

		/* This code is of no use since only one socket is watched. */
		if ( ! FD_ISSET(sock, &sockSet) )  continue;

		if ( 0 > (rv = recv(sock, &c, 1, 0)) ){
			return(-1);	/* read error */
		}
		if ( rv == 0 )  break;
		if ( c == '\r' )  continue;	/* ignore CR */
		if ( recvcount < buflen ){
			msg[recvcount++] = c;
		}
		if ( c == '\n' )  break;	/* stop at LF */
	}
	msg[recvcount] = '\0';
	return(recvcount);
}


int SendLine(int sock, char *msg, time_t timeout){
	int	rv;
	rv = SendData(sock, msg, strlen(msg), timeout);
	if ( -2 == rv ){
		fputs("Warning: Timeout occurred (send).\n",stderr);
		return(-1);
	}
	if ( 0 > rv ){
		fputs("Warning: Message transmission error.\n",stderr);
		return(-1);
	}
	return(0);
}


int SendLineLF(int sock, char *msg, time_t timeout){
	int	rv;
	int	retcode = 0;
	int	buflen;
	char	*msgbuf;
	buflen = strlen(msg) +2;
	if ( NULL == (msgbuf = (char *)malloc(buflen * sizeof(char))) ){
		fputs("Warning: Message transmission error.\n",stderr);
		return(-1);
	}
	strcpy(msgbuf, msg);
	strcat(msgbuf, "\n");
	rv = SendData(sock, msgbuf, strlen(msgbuf), timeout);
	if ( -2 == rv ){
		fputs("Warning: Timeout occurred (send).\n",stderr);
		retcode = -1;
	}
	if ( 0 > rv ){
		fputs("Warning: Message transmission error.\n",stderr);
		retcode = -1;
	}
	free((void *)msgbuf);
	return(retcode);
}


