/*--------------------------------------------------------------
    trackpeer - a sample client for peer tracking v0.01(beta)
      Written by  H.Goto, Jan 2005
      Modified by H.Goto, Mar 2005
      http://www.sc.isc.tohoku.ac.jp/~hgot/
--------------------------------------------------------------*/

/*----------------------------------------------------------------------
  Copyright (C) 2005  Hideaki Goto

  All Rights Reserved.

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files (the
  "Software"), to deal in the Software without restriction, including
  without limitation the rights to use, copy, modify, merge, publish,
  distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to
  the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR
  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name(s) of the copyright
  holder(s) shall not be used in advertising or otherwise to promote
  the sale, use or other dealings in this Software without prior
  written authorization of the copyright holder(s).
----------------------------------------------------------------------*/


#include	"trackpeerd.h"


static char	msgbuf[MAXMSGLEN];
static char	querystr[MAXMSGLEN];
static char	rescode[MAXMSGLEN];

/* flags and options */
static int	f_debug = 0;

/* internal use */
static time_t		evtime;
static in_addr_t	ip_src, ip_dst;
static uint16_t		port_src, port_dst;




/*--------------------------------------------------------------
    Command handler
--------------------------------------------------------------*/

static int cmdhandler(int sock){
	char	addrstr[30];
	int	ttl;
/* format the query command */
	sprintf(querystr,"FIND %d  %lld ", \
		DEFTTL, (long long)evtime);
	inet_ntop(AF_INET,&ip_src,addrstr,INET_ADDRSTRLEN);
	strcat(querystr,addrstr);
	sprintf(addrstr," %d ",(int)((uint)port_src));
	strcat(querystr,addrstr);
	inet_ntop(AF_INET,&ip_dst,addrstr,INET_ADDRSTRLEN);
	strcat(querystr,addrstr);
	sprintf(addrstr," %d  0.0.0.0 0",(int)((uint)port_dst));
	strcat(querystr,addrstr);

	if ( f_debug > 0 ){
		printf("Issuing the following query command.\n  %s\n",querystr);
	}

/* issue the command */
	if ( 0 > SendLineLF(sock, querystr, SENDTIMEOUT) ){
		return(-1);
	}

/* receive result lines */
	for (;;){
		if ( 0 > ReceiveLine(sock, msgbuf, MAXMSGLEN, REPLYTIMEOUT) ){
			return(-1);
		}
		printf("%s",msgbuf);
		ttl = DEFTTL;
		sscanf(msgbuf,"%s %d",rescode,&ttl);

		/* exit upon fatal error */
		if ( 0 == strcmp(rescode,"CMDERR") )  break;
		if ( 0 == strcmp(rescode,"END") && ttl == DEFTTL )  break;
	}
	return(0);
}




/*--------------------------------------------------------------
    Help message
--------------------------------------------------------------*/

static void usage(){
	fputs("trackpeer - a client for peer tracking   v0.01(beta)\n",stdout);
	fputs("Copyright (C) 2005 Hideaki Goto\n",stdout);
	fputs("usage: trackpeer [-options] -s server_addr:port\n",stdout);
	fputs("    -h             : show help message\n",stdout);
	fputs("    -v             : show debug messages\n",stdout);
	fputs("    -time TIME     : event time (secs in UTC), or 0 to search all\n",stdout);
	fputs("    -src ADDR:PORT : source IP address and port\n",stdout);
	fputs("    -dst ADDR:PORT : destination IP address and port\n",stdout);
}




/*--------------------------------------------------------------
    Main routine
--------------------------------------------------------------*/

int main(int ac, char *av[]){
	int	i,k;
	int	rcode;
	int	svSock;
	struct sockaddr_in	svAddr;
	in_addr_t	ip_srv;
	uint16_t	port_srv = TMAC_PORT;
	char	*svAddrPort_str = NULL;
	char	*srcspec_str = NULL;
	char	*dstspec_str = NULL;
	time_t	currtime;

	ip_src = ip_dst = ip_all;
	port_src = port_dst = port_all;
	evtime = -1;	/* -1: default to use current time */

	time(&currtime);	/* Current time in UTC */

	for ( k=1, i=1 ; i<ac && k ; i++ ){
		if ( 0 == strcmp("-h",av[i]) || 0 == strcmp("-help",av[i]) ){
			usage();  return(0);
		}
		if ( 0 == strcmp("-v",av[i]) ){
			f_debug = 1;  continue;
		}
		if ( 0 == strcmp("-time",av[i]) ){
			if ( i++ >= ac ){  k=0; continue; }
			if ( -1 > atoll(av[i]) ){  k=0; continue; }
			evtime = (time_t)atoll(av[i]);
			continue;
		}
		if ( 0 == strcmp("-s",av[i]) ){
			if ( i++ >= ac ){  k=0; continue; }
			svAddrPort_str = av[i];
			continue;
		}
		if ( 0 == strcmp("-src",av[i]) ){
			if ( i++ >= ac ){  k=0; continue; }
			srcspec_str = av[i];
			continue;
		}
		if ( 0 == strcmp("-dst",av[i]) ){
			if ( i++ >= ac ){  k=0; continue; }
			dstspec_str = av[i];
			continue;
		}
		k = 0;
	}
	if ( k == 0 || svAddrPort_str == NULL ){  usage();  return(0); }

	if ( evtime == -1 )  evtime = currtime;

	if ( srcspec_str ){
		if ( 0 > get_addr_port(srcspec_str, &ip_src, &port_src) ){
			fputs("Wrong source address:port specification.\n",stderr);
			return(1);
		}
	}
	if ( dstspec_str ){
		if ( 0 > get_addr_port(dstspec_str, &ip_dst, &port_dst) ){
			fputs("Wrong destination address:port specification.\n",stderr);
			return(1);
		}
	}

/* connect to the server */
	memset(&svAddr, 0, sizeof(svAddr));
	svAddr.sin_family = AF_INET;
	if ( 0 > get_addr_port(svAddrPort_str, &ip_srv, &port_srv) ){
		fputs("Wrong server specification.\n",stderr);
		return(1);
	}
	svAddr.sin_addr.s_addr = ip_srv;
	svAddr.sin_port = htons(port_srv);

	if ( f_debug > 0 ){
		printf("Connecting to server %s on port %d.\n", \
			inet_ntoa(svAddr.sin_addr),(int)((uint)port_srv));
	}

	if ( 0 > (svSock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) ){
		fputs("socket() failed.\n",stderr);
		return(1);
	}
	if ( 0 > connect(svSock, (struct sockaddr *)&svAddr, sizeof(svAddr)) ){
		fputs("connect() failed.\n",stderr);
		close(svSock);
		return(1);
	}
/* greeting */
	if ( 0 > (rcode = SendLineLF(svSock, GREETINGSTR, SENDTIMEOUT)) ){
		close(svSock);
		if ( rcode == -2 ){
			fputs("Timeout occurred (send).\n",stderr);
		}
		return(2);
	}
	if ( 0 > (rcode = ReceiveLine(svSock, msgbuf, MAXMSGLEN, REPLYTIMEOUT)) ){
		close(svSock);
		if ( rcode == -2 ){
			fputs("Timeout occurred (recv).\n",stderr);
		}
		return(2);
	}
	if ( strcmp(msgbuf, CMDVERSTRING"\n") ){
		fputs("Command version mismatch.\n",stderr);
		close(svSock);
		return(2);
	}

/* handle command */
	if ( 0 > cmdhandler(svSock) ){
		close(svSock);
		return(2);
	}

/* disconnect from the server */
	if ( 0 > SendLineLF(svSock, "BYE", SENDTIMEOUT) ){
		close(svSock);
		return(2);
	}

	close(svSock);
	return(0);
}

