/*--------------------------------------------------------------
    trackpeerd - peer tracking agent
    common definitions    trackpeerd.h
      Written by  H.Goto, Jan 2005
      Modified by H.Goto, Mar 2005
      http://www.sc.isc.tohoku.ac.jp/~hgot/
--------------------------------------------------------------*/

/*----------------------------------------------------------------------
  Copyright (C) 2005  Hideaki Goto

  All Rights Reserved.

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files (the
  "Software"), to deal in the Software without restriction, including
  without limitation the rights to use, copy, modify, merge, publish,
  distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to
  the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR
  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name(s) of the copyright
  holder(s) shall not be used in advertising or otherwise to promote
  the sale, use or other dealings in this Software without prior
  written authorization of the copyright holder(s).
----------------------------------------------------------------------*/


/*
  [commands from client]
    trackpeer : a greeting / show command version
    FIND      : find the MAC address of the client
      FIND ttl time ip_src port_src ip_dst port_dst ip_client serial
        serial is any integer value (of long long) produced by the client.
    BYE       : end of session

  [result codes from agent]
    FOUND     : address found
      FOUND ttl time flag mac_src ip_src port_src ip_dst port_dst ip_client serial
        flag is E (exact match) or L (loosely matched).
    NOTFOUND  : address not found
      NOTFOUND ttl
    FORBIDDEN : forbidden query
      FORBIDDEN ttl
    FORWARDED : query forwarded
      FORWARDED ttl ip_nextagent port_nextagent
    REDIRECTED : query redirected
      REDIRECTED ttl ip_nextagent port_nextagent
    NOROUTE   : no route
      NOROUTE ttl
    TIMEOUT   : timeout
      TIMEOUT ttl
    PROTOERR  : protocol error
      PROTOERR ttl
    END       : end of session
      END ttl

    EXPIRED   : reached TTL=0
      EXPIRED ttl
    CMDERR    : command error
      It is often impossible to know correct ttl, since this error is fatal.
      All the sessions have to be closed immediately upon this error.
*/

#ifndef	_trackpeerd_h
#define	_trackpeerd_h

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<pthread.h>
#include	<sys/socket.h>
#include	<arpa/inet.h>
#include	<sys/time.h>
#include	<time.h>
#include	<sys/types.h>
#include	<netinet/in.h>
#ifdef sun
#include	<net/if.h>
#else
#include	<netinet/ether.h>
#endif
#include	<netinet/if_ether.h>
#include	<inttypes.h>
#include	<unistd.h>

#include	"comm.h"

#define	TMAC_PORT	49200
#define	MAXTTL		30
#define	DEFTTL		4
//#define	CMDTIMEOUT	10
#define	CMDTIMEOUT	60
#define	SENDTIMEOUT	10
#define	REPLYTIMEOUT	SENDTIMEOUT
#define	MAXPENDING	5

#define	GREETINGSTR	"trackpeer"
#define	CMDVERSTRING	"trackpeer command version 1.00"
#define	MAXMSGLEN	256

//static struct ether_addr	mac_null = {{0,0,0,0,0,0}};
const in_addr_t	ip_all = 0;	/* 0.0.0.0 */
const in_addr_t	ip_null = 0;
const uint16_t	port_all = 0;

#define	maccmp(a,b)	(memcmp((void *)a,(void *)b,6))

#endif	/* _trackpeerd_h */
